/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.so.components;

import org.junit.Before;
import org.junit.Rule;
import org.junit.contrib.java.lang.system.SystemErrRule;
import org.junit.contrib.java.lang.system.SystemOutRule;
import org.junit.rules.ExpectedException;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.extension.command.monitoring.so.components.framework.AbstractMonitoredComponent;
import org.ow2.petals.cli.shell.extension.PetalsAdminShellExtension;
import org.ow2.petals.jmx.api.api.PetalsJmxApiFactory;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DuplicatedServiceException;
import org.ow2.petals.jmx.api.api.exception.MissingServiceException;
import org.ow2.petals.jmx.api.mock.PetalsJmxApiFactoryMock;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiMockTestCase;

public abstract class AbstractTest extends PetalsJmxApiMockTestCase {

    @Rule
    public final TemporaryFolder temporaryFolder = new TemporaryFolder();

    @Rule
    public final SystemOutRule systemOut = new SystemOutRule().muteForSuccessfulTests().enableLog();

    @Rule
    public final SystemErrRule systemErr = new SystemErrRule().muteForSuccessfulTests().enableLog();

    @Rule
    public final ExpectedException expectedEx = ExpectedException.none();

    protected StringStreamShell shell;

    protected AbstractMonitoredComponent component;

    @Before
    public void setUpFunction() {
        shell = new StringStreamShell();
        shell.addExtension(new PetalsAdminShellExtension());
        this.component = buildComponent();
        this.component.setShell(shell);

        // inject JMX client because petals admin does not provide the abstractions we need
        try {
            component.setJMXClient(PetalsJmxApiFactory.getInstance().createJMXClient(
                            PetalsJmxApiFactoryMock.HOST_NO_ERROR, PetalsJmxApiFactoryMock.DEFAULT_PORT,
                            PetalsJmxApiFactoryMock.DEFAULT_USERNAME, PetalsJmxApiFactoryMock.DEFAULT_PASSWORD));
        } catch (ConnectionErrorException | DuplicatedServiceException | MissingServiceException e) {
            throw new AssertionError(e);
        }
    }

    protected abstract AbstractMonitoredComponent buildComponent();
}
